<?php

/**
 * Copyright (c) 2019 Tawfek Daghistani - ConfigureTech
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

namespace Ctech\Configurator\Controller\Adminhtml\Installer;

use Magento\Framework\View\Result\PageFactory;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\Session;

class Line extends \Magento\Backend\App\Action
{

    protected $resultPageFactory;

    /** @var  \Magento\Backend\Model\Session $session  */
    protected $session;

    /**
     * 
     *
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Backend\Model\Session $session
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        Session $session
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->session = $session;
        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $keys = $this->session->getData("ctechInstaller");
        if (empty($keys)) {
            $this->messageManager->addNotice("Installer session has been expired , please start over");
            return $this->_redirect("ctechinstaller/installer/index");
        }
        $data = $this->getRequest()->getPostValue();
        if (!empty($data) && $this->getRequest()->isPost()) {
            $keys["selected_lines"] = $data["line"];
            $this->session->setData("ctechInstaller", $keys);
            $this->messageManager->addSuccess("Product Lines have been selected");
            return $this->_redirect("ctechinstaller/installer/category");
        }
        return $this->resultPageFactory->create();
    }
}
